
#include "configuration.h"
#include "extruder.h"
#include "temperature.h"
#include "OneWire.h"

extruder::extruder()
{
  pinMode(H1D, OUTPUT);
  pinMode(H1E, OUTPUT);  
  pinMode(H2D, OUTPUT);
  pinMode(H2E, OUTPUT);
  pinMode(OUTPUT_A, OUTPUT);
  pinMode(OUTPUT_B, OUTPUT);
  pinMode(OUTPUT_C, OUTPUT);
  pinMode(E_STEP_PIN, INPUT);
  pinMode(E_DIR_PIN, INPUT);  
  pinMode(POT, INPUT);
#ifdef MAX6675_THERMOCOUPLE
  pinMode(SO, INPUT);
  pinMode(SCK, OUTPUT);
  pinMode(TC_0, OUTPUT); 
  digitalWrite(TC_0,HIGH);  // Disable MAX6675
#else
#ifdef USE_DS2760
  pinMode(OWTroubleshoot, OUTPUT);
  digitalWrite(OWTroubleshoot, 0);
#else
  pinMode(TEMP_PIN, INPUT);
#endif
#endif

  disableStep();
 
#ifdef  PID_CONTROL

   pGain = TEMP_PID_PGAIN;
   iGain = TEMP_PID_IGAIN;
   dGain = TEMP_PID_DGAIN;
   temp_iState = 0;
   temp_dState = 0;
   temp_iState_min = -TEMP_PID_INTEGRAL_DRIVE_MAX/iGain;
   temp_iState_max = TEMP_PID_INTEGRAL_DRIVE_MAX/iGain;
   iState = 0;
   dState = 0;
   previousTime = millis()/MILLI_CORRECTION;

#endif
 

  // Defaults

  coilPosition = 0;  
  forward = true;
  pwmValue =  STEP_PWM;
  targetTemperature = 0;
  currentTemperature = 0;
  manageCount = 0;
  skipCount = 0;
  stp = 0;
  potVal = 0;
  potSum = 0;
  potCount = 0;
  usePot = true;
  
#ifdef PASTE_EXTRUDER
  pinMode(OPTO_PIN, INPUT); 
  valveAlreadyRunning = false;
  valveEndTime = 0;
  valveAtEnd = false;
  seenHighLow = false;
  valveState = false;
  requiredValveState = true;
  kickStartValve();
#endif
}

#ifdef  PID_CONTROL

// With thanks to Adam at Makerbot and Tim at BotHacker
// see http://blog.makerbot.com/2009/10/01/open-source-ftw/

byte extruder::pidCalculation(int dt)
{
  int output;
  int error;
  float pTerm, iTerm, dTerm;

  error = targetTemperature - currentTemperature;

  pTerm = pGain * error;

  temp_iState += error;
  temp_iState = constrain(temp_iState, temp_iState_min, temp_iState_max);
  iTerm = iGain * temp_iState;

  dTerm = dGain * (currentTemperature - temp_dState);
  temp_dState = currentTemperature;

  output = pTerm + iTerm - dTerm;
  output = constrain(output, 0, 255);

  return output;
}

#endif

void extruder::controlTemperature()
{
  currentTemperature = internalTemperature(); 
    
#ifdef PID_CONTROL

  int dt;
  unsigned long time = millis()/MILLI_CORRECTION;  // Correct for fast clock
  dt = time - previousTime;
  previousTime = time;
  if (dt > 0) // Don't do it when millis() has rolled over
    analogWrite(OUTPUT_C, pidCalculation(dt));

#else

  // Simple bang-bang temperature control

  if(targetTemperature > currentTemperature)
    digitalWrite(OUTPUT_C, 1);
  else
    digitalWrite(OUTPUT_C, 0);

#endif 
}



void extruder::slowManage()
{
  manageCount = 0;
  
  potSum += (potVoltage() >> 2);
  potCount++;
  if(potCount >= 10)
  {
    potVal = (byte)(potSum/10);
    potCount = 0;
    potSum = 0;
  }

  //blink(true);  
  
    controlTemperature();
}

void extruder::manage()
{
  byte s = digitalRead(E_STEP_PIN);
  if(s != stp)
  {
    stp = s;
    sStep(0);
  }

#ifdef PASTE_EXTRUDER
  valveMonitor();
#endif

  manageCount++;
  if(manageCount > SLOW_CLOCK)
    slowManage();   
}


/* 
 Temperature reading function  
 With thanks to: Ryan Mclaughlin - http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1230859336
 for the MAX6675 code
 */
#ifdef USE_DS2760
void outputDebugInfo(int outNum)
{
  int dataDelay = 20;
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(10);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(10);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(3);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(3);
digitalWrite(OWTroubleshoot,1);
        delayMicroseconds(3);
digitalWrite(OWTroubleshoot,0);
digitalWrite(OWTroubleshoot,bitRead(outNum,0));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,1));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,2));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,3));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
        delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,4));
        delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,5));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,6));
	delayMicroseconds(dataDelay);
 digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
  digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
  digitalWrite(OWTroubleshoot,bitRead(outNum,7));
	delayMicroseconds(dataDelay);
 digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
  digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,8));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,9));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,10));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,11));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
        delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,12));
        delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,13));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,14));
	delayMicroseconds(dataDelay);
 digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
  digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
  digitalWrite(OWTroubleshoot,bitRead(outNum,15));
	delayMicroseconds(dataDelay);
}
void outputDebugInfoByte(byte outNum)
{
  int dataDelay = 20;
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(10);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(10);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(3);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(3);
digitalWrite(OWTroubleshoot,1);
        delayMicroseconds(3);
digitalWrite(OWTroubleshoot,0);
digitalWrite(OWTroubleshoot,bitRead(outNum,0));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,1));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,2));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,3));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
        delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,4));
        delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,5));
	delayMicroseconds(dataDelay);
digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
digitalWrite(OWTroubleshoot,bitRead(outNum,6));
	delayMicroseconds(dataDelay);
 digitalWrite(OWTroubleshoot,0);
	delayMicroseconds(1);
  digitalWrite(OWTroubleshoot,1);
	delayMicroseconds(1);
  digitalWrite(OWTroubleshoot,bitRead(outNum,7));
	delayMicroseconds(dataDelay);
}
int readDS2760(int useWire)
{
  
  word vIn;
  word tmpCJ;
  word tCuV;
  byte sign;
  word cjComp;
  word tempC;
  word tblLo;
  word tblHi;
  word eePntr;
  word testVal;
  byte error;
  byte tmpRead;
  int i;
  int tries=0;
  
  OneWire myWire(useWire);
	 
  myWire.reset();
  myWire.write(SkipNet,0);
  myWire.write(RdReg,0);
  myWire.write(0x18,0);
  tmpRead=myWire.read();
  myWire.reset();
  
  if(bitRead(tmpRead,7))
  {
    tmpCJ=0;
  } else {
    tmpCJ=tmpRead;
  }	
  
  
  myWire.reset();
  myWire.write(SkipNet,0);
  myWire.write(RdReg,0);
  myWire.write(0x0E,0);
  tmpRead=myWire.read();
  tCuV=tmpRead;
  tCuV=tCuV << 8;
  tmpRead=myWire.read();
  myWire.reset();
  tCuV=tCuV | tmpRead;
  tCuV=tCuV >> 3;
  if(bitRead(tCuV,12))
  {
     tCuV = tCuV | 0xF000;
     tCuV = tCuV ^ 0xFFFF;
  }
   
 
    cjComp=temptable[(int)(tmpCJ/10)][0] + (tmpCJ-temptable[(int)(tmpCJ/10)][1])*(temptable[(int)(tmpCJ/10)+1][0]-temptable[(int)(tmpCJ/10)][0])/10;

    cjComp=cjComp+tCuV;
  if(cjComp>temptable[NUMTEMPS-1][0])
  {
    return temptable[NUMTEMPS-1][1];
  }
  for (i=1; i<NUMTEMPS; i++)
  {
    if (temptable[i][0]>cjComp)
    {
      return temptable[i-1][1] + (cjComp-temptable[i-1][0])*(temptable[i][1]-temptable[i-1][1])/(temptable[i][0]-temptable[i-1][0]);
      
    }
  }
  return 2000;  //if it fails to get a temperature return 2000 so it will turn off the heat.
}
#endif
int extruder::internalTemperature()
{
#ifdef USE_DS2760
  
  word vIn;
  word tmpCJ;
  word tCuV;
  byte sign;
  word cjComp;
  word tempC;
  word tblLo;
  word tblHi;
  word eePntr;
  word testVal;
  byte error;
  byte tmpRead;
  int tmpTemp;    //added by brnrd to fix bug in reading temp 20110308
  int i;
  int useWire;
  int tries=0;
  
  useWire=OW;
  OneWire myWire(useWire);
  do {
  OneWire myWire(useWire);
	 
  myWire.reset();
  myWire.write(SkipNet,0);
  myWire.write(RdReg,0);
  myWire.write(0x18,0);
  tmpRead=myWire.read();
  myWire.reset();
  
  if(bitRead(tmpRead,7))
  {
    tmpCJ=0;
  } else {
    tmpCJ=tmpRead;
  }	
    if(tmpCJ==0)
    {
      tries+=1;
      if(useWire==OW)
      {
         useWire=OWTroubleshoot;
      } else {
        useWire=OW;
      }
    } else {
      tries=10;
    }
	} while (tries<5);
  
 if(tries<10)
 {
   return 2000;
 }
  
 // tmpRead=readDS2760(useWire);
 tmpTemp=readDS2760(useWire);    // but fixed by brnrd 2011/3/08, was assigning this to a byte and truncating the temperature reading to 8-bits (max=255) leading to overheating
  
//  if(tmpRead==0)
  if(tmpTemp==0)
  {
    return 9999;
  } else {
//    return tmpRead;
     return tmpTemp;
  }
  
  myWire.reset();
  myWire.write(SkipNet,0);
  myWire.write(RdReg,0);
  myWire.write(0x0E,0);
  tmpRead=myWire.read();
  tCuV=tmpRead;
  tCuV=tCuV << 8;
  tmpRead=myWire.read();
  myWire.reset();
  tCuV=tCuV | tmpRead;
  tCuV=tCuV >> 3;
  if(bitRead(tCuV,12))
  {
     tCuV = tCuV | 0xF000;
     tCuV = tCuV ^ 0xFFFF;
  }
   
 
    cjComp=temptable[(int)(tmpCJ/10)][0] + (tmpCJ-temptable[(int)(tmpCJ/10)][1])*(temptable[(int)(tmpCJ/10)+1][0]-temptable[(int)(tmpCJ/10)][0])/10;

    cjComp=cjComp+tCuV;
  if(cjComp>temptable[NUMTEMPS-1][0])
  {
    return temptable[NUMTEMPS-1][1];
  }
  for (i=1; i<NUMTEMPS; i++)
  {
    if (temptable[i][0]>cjComp)
    {
      return temptable[i-1][1] + (cjComp-temptable[i-1][0])*(temptable[i][1]-temptable[i-1][1])/(temptable[i][0]-temptable[i-1][0]);
      
    }
  }
  return 2000;  //if it fails to get a temperature return 2000 so it will turn off the heat.
#endif
#ifdef USE_THERMISTOR
  int raw = analogRead(TEMP_PIN);

  int celsius = raw;
  byte i;

  // TODO: This should do a binary chop

  for (i=1; i<NUMTEMPS; i++)
  {
    if (temptable[i][0] > raw)
    {
      celsius  = temptable[i-1][1] + 
        (raw - temptable[i-1][0]) * 
        (temptable[i][1] - temptable[i-1][1]) /
        (temptable[i][0] - temptable[i-1][0]);

      break;
    }
  }

  // Overflow: Set to last value in the table
  if (i == NUMTEMPS) celsius = temptable[i-1][1];
  // Clamp to byte
  if (celsius > 255) celsius = 255; 
  else if (celsius < 0) celsius = 0; 

  return celsius;
#endif

#ifdef AD595_THERMOCOUPLE
  return ( 5.0 * analogRead(TEMP_PIN) * 100.0) / 1024.0; //(int)(((long)500*(long)analogRead(TEMP_PIN))/(long)1024);
#endif  

#ifdef MAX6675_THERMOCOUPLE
  skipCount++;
  if (skipCount>SKIP_CLOCK_COUNT)
  {
    skipCount=0;
  return currentTemperature;
  }
else
{
  int value = 0;
  byte error_tc;


  digitalWrite(TC_0, 0); // Enable device

  /* Cycle the clock for dummy bit 15 */
  digitalWrite(SCK,1);
  digitalWrite(SCK,0);

  /* Read bits 14-3 from MAX6675 for the Temp
   	 Loop for each bit reading the value 
   */
  for (int i=11; i>=0; i--)
  {
    digitalWrite(SCK,1);  // Set Clock to HIGH
    value += digitalRead(SO) << i;  // Read data and add it to our variable
    digitalWrite(SCK,0);  // Set Clock to LOW
  }

  /* Read the TC Input inp to check for TC Errors */
  digitalWrite(SCK,1); // Set Clock to HIGH
  error_tc = digitalRead(SO); // Read data
  digitalWrite(SCK,0);  // Set Clock to LOW

  digitalWrite(TC_0, 1); //Disable Device

  if(error_tc)
    return 2000;
  else
    return value/4;

}
#endif
}

void extruder::waitForTemperature()
{

}

void extruder::valveSet(bool closed)
{
#ifdef PASTE_EXTRUDER
  requiredValveState = closed;
  kickStartValve();
#endif
}

void extruder::setDirection(bool direction)
{
  forward = direction;  
}

void extruder::setCooler(byte e_speed)
{
  analogWrite(OUTPUT_B, e_speed);   
}

void extruder::setTemperature(int tp)
{
  targetTemperature = tp;
}

int extruder::getTemperature()
{
  return currentTemperature;  
}

void extruder::sStep(byte dir)
{
#ifndef PASTE_EXTRUDER
  byte pwm;
  
  if(usePot)
    pwm = potVal;
  else
    pwm = pwmValue;

  // This increments or decrements coilPosition then writes the appropriate pattern to the output pins.

  switch(dir)
  {
    case 1:
      coilPosition++;
      break;
      
    case 2:
      coilPosition--;
      break;
      
    default:
      if(digitalRead(E_DIR_PIN))
        coilPosition++;
      else
        coilPosition--;
      break;
  }
  
  coilPosition &= 7;

  // Which of the 8 possible patterns do we want?
  // The pwm = (pwm >> 1) + (pwm >> 3); lines
  // ensure (roughly) equal power on the half-steps

#ifdef FULL_STEP
  switch((coilPosition&3) << 1)
#else
  switch(coilPosition)
#endif 
  {
  case 7:
    pwm = (pwm >> 1) + (pwm >> 3);
    digitalWrite(H1D, 1);    
    digitalWrite(H2D, 1);
    analogWrite(H1E, pwm);
    analogWrite(H2E, pwm);    
    break;

  case 6:
    digitalWrite(H1D, 1);    
    digitalWrite(H2D, 1);
    analogWrite(H1E, pwm);
    analogWrite(H2E, 0);   
    break; 

  case 5:
    pwm = (pwm >> 1) + (pwm >> 3);
    digitalWrite(H1D, 1);
    digitalWrite(H2D, 0);
    analogWrite(H1E, pwm);
    analogWrite(H2E, pwm); 
    break;

  case 4:
    digitalWrite(H1D, 1);
    digitalWrite(H2D, 0);
    analogWrite(H1E, 0);
    analogWrite(H2E, pwm); 
    break;

  case 3:
    pwm = (pwm >> 1) + (pwm >> 3);
    digitalWrite(H1D, 0);
    digitalWrite(H2D, 0);
    analogWrite(H1E, pwm);
    analogWrite(H2E, pwm); 
    break; 

  case 2:
    digitalWrite(H1D, 0);
    digitalWrite(H2D, 0);
    analogWrite(H1E, pwm);
    analogWrite(H2E, 0); 
    break;

  case 1:
    pwm = (pwm >> 1) + (pwm >> 3);
    digitalWrite(H1D, 0);
    digitalWrite(H2D, 1);
    analogWrite(H1E, pwm);
    analogWrite(H2E, pwm); 
    break;

  case 0:
    digitalWrite(H1D, 0);
    digitalWrite(H2D, 1);
    analogWrite(H1E, 0);
    analogWrite(H2E, pwm); 
    break; 

  }
#endif
}


void extruder::enableStep()
{
  // Nothing to do here - step() automatically enables the stepper drivers appropriately.  
}

void extruder::disableStep()
{
  analogWrite(H1E, 0);
  analogWrite(H2E, 0);  
}

int extruder::potVoltage()
{
  return (int)analogRead(POT);  
}

void extruder::setPWM(int p)
{
  pwmValue = p;
  usePot = false;
  sStep(1);
  sStep(2);
}

void extruder::usePotForMotor()
{
  usePot = true;
  sStep(1);
  sStep(2);
}

char* extruder::processCommand(char command[])
{
  reply[0] = 0;
  switch(command[0])
  {
  case WAIT_T:
    waitForTemperature();
    break;

  case VALVE:
    valveSet(command[1] != '1');
    break;

  case DIRECTION:
    // setDirection(command[1] == '1'); // Now handled by hardware.
    break;

  case COOL:
    setCooler(atoi(&command[1]));
    break;

  case SET_T:
    setTemperature(atoi(&command[1]));
    break;

  case GET_T:
    itoa(getTemperature(), reply, 10);
    break;

  case STEP:
    //sStep(0); // Now handled by hardware.
    break;

  case ENABLE:
    enableStep();
    break;

  case DISABLE:
    disableStep();
    break;

  case PREAD:
    itoa(potVoltage(), reply, 10);
    break;

  case SPWM:
    setPWM(atoi(&command[1]));
    break;

  case UPFM:
    usePotForMotor();
    break;  

  case PING:
    break;

  default:
    return 0; // Flag up dud command
  }
  return reply; 
}

#ifdef PASTE_EXTRUDER

bool extruder::valveTimeCheck(int millisecs)
{
  if(valveAlreadyRunning)
  {
    if(millis() >= valveEndTime)
    {
      valveAlreadyRunning = false;
      return true;
    }
    return false;
  }

// MILLI_CORRECTION not needed here; makes time resolution too coarse
  valveEndTime = millis() + millisecs; //*MILLI_CORRECTION;
  valveAlreadyRunning = true;
  return false;
}

void extruder::valveTurn(bool close)
{
  if(valveAtEnd)
    return;
    
  byte valveRunningState = VALVE_STARTING;
  if(digitalRead(OPTO_PIN))
  {
    seenHighLow = true;
    valveRunningState = VALVE_RUNNING;
  } else
  {
    if(!seenHighLow)
     valveRunningState = VALVE_STARTING;
    else
     valveRunningState = VALVE_STOPPING; 
  }    
   
  switch(valveRunningState)
  {
  case VALVE_STARTING: 
          if(close)
             digitalWrite(H1D, 1);
          else
             digitalWrite(H1D, 0);
          digitalWrite(H1E, HIGH);
          break;
          
  case VALVE_RUNNING:
          return;
  
  case VALVE_STOPPING:
          if(close)
            digitalWrite(H1D, 0);
          else
            digitalWrite(H1D, 1);
            
          if(!valveTimeCheck(10))
            return;
            
          digitalWrite(H1E, LOW);
          valveState = close;
          valveAtEnd = true;
          seenHighLow = false;
          break;
          
  default:
          break;
  }  
}

void extruder::valveMonitor()
{
  if(valveState == requiredValveState)
    return;
  valveAtEnd = false;
  valveTurn(requiredValveState);
} 

void extruder::kickStartValve()
{
  if(digitalRead(OPTO_PIN))
  {
     if(requiredValveState)
       digitalWrite(H1D, 1);
     else
       digitalWrite(H1D, 0);
     digitalWrite(H1E, HIGH);    
  }
} 
#endif
   
   


